//
// This code is part of https://developer.mescius.com/document-solutions/dot-net-pdf-api/demos.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.IO;
using System.Drawing;
using System.Numerics;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Text;
using GCTEXT = GrapeCity.Documents.Text;
using GCDRAW = GrapeCity.Documents.Drawing;

namespace DsPdfWeb.Demos
{
    // This example shows how to use GcGraphics.PushTransparencyLayer()/PopTransparencyLayer() methods
    // to specify a common opacity for a group of drawing operations.
    // The code is similar to AddWatermark sample, but instead of using a semi-transparent
    // text color, it performs all related drawing on a transparency layer.
    // This provides a more flexible and convenient approach to control transparency.
    public class TransparencyLayer
    {
        public int CreatePDF(Stream stream)
        {
            var doc = new GcPdfDocument();
            using var fs = File.OpenRead(Path.Combine("Resources", "PDFs", "SlidePages.pdf"));
            doc.Load(fs);
            foreach (var page in doc.Pages)
            {
                var g = page.Graphics;

                // Text layout used to draw the 'watermark':
                var tl = g.CreateTextLayout();
                tl.Append("DsPdf Demo");
                tl.DefaultFormat.Font = GCTEXT.Font.FromFile(Path.Combine("Resources", "Fonts", "calibrib.ttf"));
                tl.DefaultFormat.FontSize = g.Resolution;
                tl.DefaultFormat.ForeColor = Color.Blue;
                tl.DefaultFormat.GlyphAdvanceFactor = 1.5f;
                tl.PerformLayout();

                // Rotation angle (radians) - from left/bottom to right/top corners of the page:
                var angle = -Math.Asin(g.CanvasSize.Width / g.CanvasSize.Height);
                // Page center:
                var center = new PointF(g.CanvasSize.Width / 2, g.CanvasSize.Height / 2);
                // Additional offset from text size:
                var delta = new PointF(
                    (float)((tl.ContentWidth * Math.Cos(angle) - tl.ContentHeight * Math.Sin(angle)) / 2),
                    (float)((tl.ContentWidth * Math.Sin(angle) + tl.ContentHeight * Math.Cos(angle)) / 2));

                // Create semi-opaque transparency layer. All subsequent drawing operations
                // will be drawn using the specified opacity till the PopTransparencyLayer() call:
                g.PushTransparencyLayer(null, 0.5f);

                // Draw watermark text diagonally in the center of the page
                // (matrix transforms are applied from last to first):
                g.Transform =
                    Matrix3x2.CreateRotation((float)angle) *
                    Matrix3x2.CreateTranslation(center.X - delta.X, center.Y - delta.Y);
                g.DrawTextLayout(tl, PointF.Empty);
                // Add a border around the text, it will also be drawn semi-transparently:
                var rc = new RectangleF(-30, -20, tl.ContentWidth + 60, tl.ContentHeight + 40);
                g.DrawRoundRect(rc, 30, new GCDRAW.Pen(Color.DarkOrange, 8));
                g.Transform = Matrix3x2.Identity;

                // Pop transparency layer:
                g.PopTransparencyLayer();
            }
            doc.Save(stream);
            return doc.Pages.Count;
        }
    }
}
